<?php
if(!isset($_YUNICC)) { die(); } // Protect against direct access.
/* This include contains functions related to string handling. */

function ends_with($haystack, $needle)
{
	if(substr($haystack, strlen($haystack) - strlen($needle), strlen($needle)) == $needle)
	{
		return true;
	}
	else
	{
		return false;
	}
}

function split_if_present($haystack, $needle)
{
	if(strpos($haystack, $needle) !== false)
	{
		return explode($needle, $haystack);
	}
	else
	{
		return false;
	}
}

function split_lines($input)
{
	return explode("\n", str_replace("\r", "", $input));
}

function arraytolower($array)
{ 
	return unserialize(strtolower(serialize($array))); 
} 

function clean_tag($tag)
{
	return preg_replace("/[^a-zA-Z0-9']/", "", normalize_chars($tag));
}

/* Thanks to highstrike at gmail dot com (http://www.php.net/manual/en/function.substr.php#80247) */
function cut_text($value, $length)
{    
    if(is_array($value)) list($string, $match_to) = $value;
    else { $string = $value; $match_to = $value{0}; }

    $match_start = stristr($string, $match_to);
    $match_compute = strlen($string) - strlen($match_start);

    if (strlen($string) > $length)
    {
        if ($match_compute < ($length - strlen($match_to)))
        {
            $pre_string = substr($string, 0, $length);
            $pos_end = strrpos($pre_string, " ");
            if($pos_end === false) $string = $pre_string."...";
            else $string = substr($pre_string, 0, $pos_end)."...";
        }
        else if ($match_compute > (strlen($string) - ($length - strlen($match_to))))
        {
            $pre_string = substr($string, (strlen($string) - ($length - strlen($match_to))));
            $pos_start = strpos($pre_string, " ");
            $string = "...".substr($pre_string, $pos_start);
            if($pos_start === false) $string = "...".$pre_string;
            else $string = "...".substr($pre_string, $pos_start);
        }
        else
        {        
            $pre_string = substr($string, ($match_compute - round(($length / 3))), $length);
            $pos_start = strpos($pre_string, " "); $pos_end = strrpos($pre_string, " ");
            $string = "...".substr($pre_string, $pos_start, $pos_end)."...";
            if($pos_start === false && $pos_end === false) $string = "...".$pre_string."...";
            else $string = "...".substr($pre_string, $pos_start, $pos_end)."...";
        }

        $match_start = stristr($string, $match_to);
        $match_compute = strlen($string) - strlen($match_start);
    }
    
    return $string;
}

function normalize_chars($str)
{
	$charmap = array(
		'Š'=>'S', 'š'=>'s', 'Ð'=>'Dj','Ž'=>'Z', 'ž'=>'z', 'À'=>'A', 'Á'=>'A', 'Â'=>'A', 'Ã'=>'A', 'Ä'=>'A', 
		'Å'=>'A', 'Æ'=>'A', 'Ç'=>'C', 'È'=>'E', 'É'=>'E', 'Ê'=>'E', 'Ë'=>'E', 'Ì'=>'I', 'Í'=>'I', 'Î'=>'I', 
		'Ï'=>'I', 'Ñ'=>'N', 'Ò'=>'O', 'Ó'=>'O', 'Ô'=>'O', 'Õ'=>'O', 'Ö'=>'O', 'Ø'=>'O', 'Ù'=>'U', 'Ú'=>'U', 
		'Û'=>'U', 'Ü'=>'U', 'Ý'=>'Y', 'Þ'=>'B', 'ß'=>'Ss','à'=>'a', 'á'=>'a', 'â'=>'a', 'ã'=>'a', 'ä'=>'a', 
		'å'=>'a', 'æ'=>'a', 'ç'=>'c', 'è'=>'e', 'é'=>'e', 'ê'=>'e', 'ë'=>'e', 'ì'=>'i', 'í'=>'i', 'î'=>'i', 
		'ï'=>'i', 'ð'=>'o', 'ñ'=>'n', 'ò'=>'o', 'ó'=>'o', 'ô'=>'o', 'õ'=>'o', 'ö'=>'o', 'ø'=>'o', 'ù'=>'u', 
		'ú'=>'u', 'û'=>'u', 'ý'=>'y', 'ý'=>'y', 'þ'=>'b', 'ÿ'=>'y', 'ƒ'=>'f', 'č'=>'c'
	);
	
	return strtr($str, $charmap);
}

function url_name($str)
{
	$str = preg_replace("/-\s/", "-", $str);
	$str = preg_replace("/[^a-z0-9\-\s]/i", "", $str);
	$str = preg_replace("/\s/", "-", $str);
	$str = strtolower($str);
	return $str;
}

function rgb_from_hex($hex)
{
	if(strlen($hex) == 6)
	{
		$r = substr($hex, 0, 2);
		$g = substr($hex, 2, 2);
		$b = substr($hex, 4, 2);
		
		//echo("{$hex} // {$r} // {$g} // {$b}");
		
		$rgb['r'] = base_convert($r, 16, 10);
		$rgb['g'] = base_convert($g, 16, 10);
		$rgb['b'] = base_convert($b, 16, 10);
		
		//var_dump($rgb);
		
		return $rgb;
	}
	else
	{
		return false;
	}
}

function hex_from_rgb($rgb)
{
	if(!empty($rgb['r']) && !empty($rgb['g']) && !empty($rgb['b']))
	{
		return base_convert($rgb['r'], 10, 16) . base_convert($rgb['g'], 10, 16) . base_convert($rgb['b'], 10, 16);
	}
	else
	{
		return false;
	}
}

function hsv_from_rgb ($rgb)
{
	if(!empty($rgb['r']) && !empty($rgb['g']) && !empty($rgb['b']))
	{
		$hsv = array();
		
		$var_r = $rgb['r'] / 255;
		$var_g = $rgb['g'] / 255;
		$var_b = $rgb['b'] / 255;
		
		$var_min = min($var_r, $var_g, $var_b);
		$var_max = max($var_r, $var_g, $var_b);
		$del_max = $var_max - $var_min;
		
		$hsv['v'] = $var_max;
		
		if($del_max == 0)
		{
			$hsv['h'] = 0;
			$hsv['s'] = 0;
		}
		else
		{
			$hsv['s'] = $del_max / $var_max;
			
			$del_r = ((($var_max - $var_r) / 6) + ($del_max / 2)) / $del_max;
			$del_g = ((($var_max - $var_g) / 6) + ($del_max / 2)) / $del_max;
			$del_b = ((($var_max - $var_b) / 6) + ($del_max / 2)) / $del_max;
			
			if($var_r == $var_max)
			{
				$hsv['h'] = $del_b - $del_g;
			}
			elseif($var_g == $var_max)
			{
				$hsv['h'] = (1 / 3) + $del_r - $del_b;
			}
			elseif($var_b == $var_max)
			{
				$hsv['h'] = (2 / 3) + $del_g + $del_r;
			}
			
			if($hsv['h'] < 0)
			{
				$hsv['h']++;
			}
			
			if($hsv['h'] > 1)
			{
				$hsv['h']--;
			}
		}
	}
	
	return $hsv;
}
?>
